#include "printf.h"
#include "as31glue.h"
#include "parse.h"	// for event codes
#include "string.h"
#include "stricmp.h"
#include "sta013.h"
#include "main.h"

#include "display.h"

#define SCROLL_SPEED 5
#define VOL_TIMEOUT 50
#define SCREEN_TIMEOUT 50

#define SCREEN1		0x01
#define SCREEN2		0x02
#define SCREEN3		0x03
#define SCREENMAX	0x03

#define CONTROL0	0x00
#define CONTROL1	0x01
#define CONTROL2	0x02
#define CONTROL3	0x03
#define CONTROL4	0x04
#define CONTROL5	0x05
#define CONTROL6	0x06
#define CONTROL7	0x07
#define CONTROL8	0x08

xdata int xwrap=0;		// Position inside filebuf
xdata unsigned char xscroll=0;	// Scroll ?

xdata unsigned char filebuf[128];
xdata unsigned char displaybuf[25]="                        ";

data unsigned char filebuf_len=0;


/* when we switch to navigation with the arrows, this "state" */
/* will probably be replaced with two variables... the current */
/* screen that the user is viewing, and the currently selected */
/* item on that screen */

#define CUR_VOLUME      1
#define CUR_TREBLE      2
#define CUR_BASS        3

xdata unsigned char current_button;
xdata unsigned char current_screen=SCREEN1;
xdata unsigned char current_control=CONTROL0;




void lcd_bass_set(char bass);
void lcd_treble_set(char treble);
void lcd_clear_bottom_row(void);
void lcd_volume_set(unsigned char attn);
void lcd_volume_set_db(unsigned char attn, unsigned char x, unsigned char y);
void lcd_update_top_line(void);




//
// Timer usage:
//
// timer 0 - scrolling song name
// timer 1 - screen display timeout
// timer 2
// timer 3
//






/* handle user interface events that related to navigation of */
/* the on-screen information */

void lcd_user_action(unsigned char event)
{
	switch(event) {

	  case E_INC:
		switch(current_button) {
		  case CUR_VOLUME:
			action_vol_up();
			break;
		  case CUR_TREBLE:
			action_treble_up();
			break;
		  case CUR_BASS:
			action_bass_up();
			break;
		  default:
			current_button=CUR_VOLUME;
			break;
		}
		break;

	  case E_DEC:
		switch(current_button) {
		  case CUR_VOLUME:
			action_vol_down();
			break;
		  case CUR_TREBLE:
			action_treble_down();
			break;
		  case CUR_BASS:
			action_bass_down();
			break;
		  default:
			current_button=CUR_VOLUME;
			break;
		}
		break;

	  case E_RIGHT:
		current_control++;
		lcd_change_menu(0);
		break;

	  case E_LEFT:
		current_control--;
		lcd_change_menu(0);
		break;

	  case E_ENTER:
		if(current_control==CONTROL0)
			lcd_change_menu(1);
		if(current_control==CONTROL1)
			put_back_event(E_PLAY_PAUSE);
		if(current_control==CONTROL2)
			put_back_event(E_RANDOM);
		if(current_control==CONTROL3)
			put_back_event(E_NEXT);
		break;

#if 0
	  case 9999:
		current_button++;
		if (current_button > 3) current_button=1;
		switch(current_button) {
		  case CUR_VOLUME:
			lcd_volume_set(setting.attn);
			break;
		  case CUR_TREBLE:
			lcd_treble_set(setting.treble);
			break;
		  case CUR_BASS:
			lcd_bass_set(setting.bass);
			break;
		}
		break;
#endif

	  case E_TIMER1:
		current_screen=SCREEN1;
		current_control=CONTROL0;
		lcd_change_menu(0);
		break;

	  default:
		printf("Unsupported LCD Navigation, #%d\r\n", event);
	}
}





void lcd_display_filename(xdata char *filename)
//
//	lcd_display_filename
//
//	Takes long file name.  Strips last 4 characters if
//	they are ".mp3".  Pads with a few spaces and
//	fills to 24 characters if shorter than that.
//
{
	unsigned char z;
	xdata char *p;

	// Kill any pending timer0 interrupts
	clear_timer(0);
	xscroll=0;

	// strcpy(filebuf,filename);
	// len = strlen(filebuf);

	filebuf_len = 0;
	p = filebuf;
	while (*filename) {
		*p++ = *filename++;
		filebuf_len++;
	}
	*p = '\0';

	if (filebuf_len >= 4 && stricmp(&filebuf[filebuf_len-4],".MP3") == 0)
	{
		filebuf_len -= 4;
		filebuf[filebuf_len]='\0';
	}

	for(z=0; z<filebuf_len; z++)
		if (filebuf[z]=='_')
			filebuf[z]=' ';

	if (filebuf_len < 24)
	{
		for (z=filebuf_len; z<24; z++) {
			filebuf[z] = ' ';
		}
		filebuf[24] = '\0';
		filebuf_len = 24;
		xscroll=0;
	}
	else
	{
		strcat(filebuf,"   ");
		filebuf_len += 3;
		xscroll=1;
	}

	for(z=0;z<24;z++) displaybuf[z]=filebuf[z];
	displaybuf[24]='\0';
	xwrap=23;

	printf("\\[\\A0\\B !%s\\]",displaybuf);

	// Set timer to update display
	set_timer(0,SCROLL_SPEED);

	// Assume this is the beginning of a new file and the
	// playback mode isn't know yet
	playback.mode_known = 0;
}





void lcd_update_2nd_line(void)
//
//	lcd_update_2nd_line
//
//	Called when timer 0 expires.
//	Displays displaybuf on the second line of the display, then
//	scrolls displaybuf to the left, and grabs the next
//	character from filebuf.
//
{
	unsigned char z,f;

	f=32;
	if(current_control==CONTROL3)
		f=33;

	if (xscroll != 0)
	{
		xwrap++;
		if(xwrap >= filebuf_len) xwrap=0;

		for(z=0;z<23;z++) displaybuf[z]=displaybuf[z+1];

		displaybuf[23]=filebuf[xwrap];

	}
	if((xscroll!=0)||(f==33))
		printf("\\[\\C%c\\B !%s\\C \\]\r\n",(char)f,displaybuf);

	set_timer(0,SCROLL_SPEED);
}





void lcd_screen_dynamic(void)
//
//	lcd_screen_dynamic
//
//	Update dynamic data.
//
{
	switch(current_screen)
	{
	  case SCREEN1:
		// TODO: only send this if it has changed from the last time */
		update_playback_parms();
		if (playback.mode_known) {
			printf("\\[\\B &%3d kbps  %2d kHz  %s\\]\r\n",
				playback.bitrate, playback.freq, playback.mode);
		} else {
			printf("\\[\\B &    kbps     kHz        \\]\r\n");
		}
		lcd_volume_set_db(setting.attn,0,7);
		break;

	  case SCREEN2:
		lcd_treble_set(setting.treble);
		printf("\\[\\B $Treble freq: 3000Hz\\]\r\n");
		lcd_bass_set(setting.bass);
		printf("\\[\\B &Bass freq:    160Hz\\]\r\n");
		break;

	  case SCREEN3:
		break;

	  default:
		break;
	}
}





void lcd_change(unsigned char parm)
{
	switch (parm) {

	  case LCD_VOLUME:
		lcd_volume_set_db(setting.attn,0,7);
		return;

	  case LCD_TREBLE:
		lcd_treble_set(setting.treble);
		return;

	  case LCD_BASS:
		lcd_bass_set(setting.bass);
		return;

	  case LCD_PLAY_MODE:
	  case LCD_RANDOM_MODE:
		lcd_update_top_line();
		return;

	  default:
		printf("Unknown LCD change event: %d\n", parm);
	}
}



void lcd_update_top_line(void)
{
	if (playing) {
		lcd_navigation(0,0,"Playing ",CONTROL1);
	} else {
		lcd_navigation(0,0,"Paused  ",CONTROL1);
	}
	if (rand8_seed) {
		lcd_navigation(8,0," Random   ",CONTROL2);
	} else {
		lcd_navigation(8,0," Normal   ",CONTROL2);
	}
	printf("\\[\\B2 ---:--\\]\r\n");
}





void lcd_volume_set(unsigned char attn)
//
//	lcd_volume_set
//
//	Displays a volume bargraph on the bottom line of
//	the display.
//
{
	unsigned char z,x;

	// Make the bargraph fit in 17 characters
	x=(0x40-attn)/4;

	printf("\\[\\A1\\B 'Vol: [");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			printf("\\C!\\w\\C ");
		}
		else
		{
			printf("-");
		}
	}
	printf("]\\]");
	set_timer(1,50);
}





void lcd_volume_set_db(unsigned char attn,unsigned char x, unsigned char y)
//
//	lcd_volume_set_db
//
//	Displays a volume db at the specified coordinates.
//
{
	lcd_navigation(x,y,"Vol: ",CONTROL5);

	if(attn==0)
	{
		printf("\\[\\A1\\B%%'  0 dB\\]\r\n");
	}
	else
	{
		printf("\\[\\A1\\B%%'-%2d dB\\]\r\n",attn);
	}
}





void lcd_bass_set(char bass)
//
//	lcd_bass_set
//
//	Displays a bass bargraph on the bottom line of
//	the display.
//
//	The sta013 expects a 2s compliment number from
//	-12 to 12 corresponding to a bass boost of
//	-18dB to 18dB.
//
{
	unsigned char z,x;

	// Make the bargraph fit in 17 characters
	x=(bass+12)*17/25;

	printf("\\[\\A1\\B %%Bas: [");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			printf("\\C!\\w\\C ");
		}
		else
		{
			printf("-");
		}
	}
	printf("]\\]\r\n");
}





void lcd_treble_set(char treble)
//
//	lcd_treble_set
//
//	Displays a treble bargraph on the bottom line of
//	the display.
//
//	The sta013 expects a 2s compliment number from
//	-12 to 12 corresponding to a treble boost of
//	-18dB to 18dB.
//
{
	unsigned char z,x;

	// Make the bargraph fit in 17 characters
	x=(treble+12)*17/25;

	printf("\\[\\A1\\B #Tre: [");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			printf("\\C!\\w\\C ");
		}
		else
		{
			printf("-");
		}
	}
	printf("]\\]\r\n");
}





void lcd_clear_bottom_row(void)
//
//	lcd_clear_bottom_row
//
//	Erases all text on the bottom row of the
//	display.  Set current button to nothing.
//
{
	printf("\\[\\A1\\B '                        \\]");
	current_button=CUR_VOLUME;
}





void lcd_change_menu(unsigned char action)
//
//	lcd_change_menu
//
//	Clears display and displays the initial data
//	for the current screen.  Increments current screen
//	if action==1;
//
{
	xdata unsigned char bounds[]={0,5,5,0};

	// Some sanity checks
	if(action==1)
	{
		if((++current_screen)>SCREENMAX)
			current_screen=SCREEN1;
	}

	if(current_control>bounds[current_screen]) current_control=CONTROL0;
	if(current_control<CONTROL0) current_control=bounds[current_screen];

	printf("\\[\\@ 0\\]\r\n");	// Clear display

	switch(current_screen)
	{
	  case SCREEN1:
		lcd_update_top_line();
		lcd_update_2nd_line();
		lcd_screen_dynamic();
		lcd_navigation(14,7,"  Config1>",CONTROL0);
//		set_timer(1,SCREEN_TIMEOUT);
		break;

	  case SCREEN2:
		lcd_update_top_line();
		lcd_update_2nd_line();
		lcd_screen_dynamic();
		lcd_navigation(14,7,"  Config2>",CONTROL0);
		set_timer(1,SCREEN_TIMEOUT);
		break;

	  case SCREEN3:
		lcd_update_top_line();
		lcd_update_2nd_line();
		lcd_screen_dynamic();
		lcd_navigation(14,7,"     Main>",CONTROL0);
		set_timer(1,SCREEN_TIMEOUT);
		break;

	  default:
		break;
	}
}





void lcd_navigation(unsigned char x, unsigned char y,
	char *navtext, unsigned char this_control)
//
//	lcd_navigation
//
//	Display navigation element on the screen at coordinates (x,y).
//	If this_control is the active one then display in bold.
//
{
	if(this_control==current_control)
	{
		printf("\\[\\A1\\B%c%c\\C!%s\\C \\]\r\n",(char)(x+32),(char)(y+32),navtext);
	}
	else
	{
		printf("\\[\\A1\\B%c%c\\C %s\\C \\]\r\n",(char)(x+32),(char)(y+32),navtext);
	}
}





void lcd_setup(void)
{
	printf("lcd:");
	printf("\\H0INC         ");	// button 0 down = INC
	printf("\\H@            ");	// button 0 up = <nothing>
	printf("\\HPINC         ");	// button 0 repeat = INC

	printf("\\H1DEC         ");	// button 1 down = INC
	printf("\\HA            ");	// button 1 up = <nothing>
	printf("\\HQDEC         ");	// button 1 repeat = INC

	printf("\\H2PRV_LIST    ");	// button 2 down = PRV_PLAYLIST
	printf("\\HB            ");	// button 2 up = <nothing>
	printf("\\HRPRV_LIST    ");	// button 2 repeat = PRV_PLAYLIST

	printf("\\H3NXT_LIST    ");	// button 3 down = NXT_PLAYLIST
	printf("\\HC            ");	// button 3 up = <nothing>
	printf("\\HSNXT_LIST    ");	// button 3 repeat = NXT_PLAYLIST

	printf("\\H4            ");	// button 4 down = <nothing>
	printf("\\HDPREVIOUS    ");	// button 4 up = PREVIOUS
	printf("\\HTREVERSE     ");	// button 4 repeat = REVERSE

	printf("\\H5PLAY        ");	// button 5 down = PLAY
	printf("\\HE            ");	// button 5 up = <nothing>
	printf("\\HU            ");	// button 5 repeat = <nothing>

	printf("\\H6            ");	// button 6 down = <nothing>
	printf("\\HFNEXT        ");	// button 6 up = NEXT
	printf("\\HVFORWARD     ");	// button 6 repeat = FORWARD

	printf("\\H7UP          ");	// button 7 down = UP
	printf("\\HG            ");	// button 7 up = <nothing>
	printf("\\HWUP          ");	// button 7 repeat = UP

	printf("\\H8LEFT        ");	// button 8 down = LEFT
	printf("\\HH            ");	// button 8 up = <nothing>
	printf("\\HXLEFT        ");	// button 8 repeat = LEFT

	printf("\\H9RIGHT       ");	// button 9 down = RIGHT
	printf("\\HI            ");	// button 9 up = <nothing>
	printf("\\HYRIGHT       ");	// button 9 repeat = RIGHT

	printf("\\H:DOWN        ");	// button 10 down = DOWN
	printf("\\HJ            ");	// button 10 up = <nothing>
	printf("\\HZDOWN        ");	// button 10 repeat = DOWN

	printf("\\H;ENTER       ");	// button 11 down = ENTER
	printf("\\HK            ");	// button 11 up = <nothing>
	printf("\\H[ENTER       ");	// button 11 repeat = ENTER

	printf("\r\n");

	current_button = CUR_VOLUME;

}


